/*
 * FPMs-TCTS SOFTWARE LIBRARY
 *
 * File:    database_old.h
 * Time-stamp: <2000-03-21 00:13:35 vincent>
 * Purpose: Decode raw formats before 2.05 release, here for compatibility purpose
 *          Use pretty much RAW functions
 * Author: Vincent Pagel
 * 
 * Copyright (c) 1995-2018 Faculte Polytechnique de Mons (TCTS lab)
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * History:
 *
 *  25/06/98 : Created from 2.06 code chunks
 */

#ifndef _DATABASE_OLD_H
#define _DATABASE_OLD_H

Database* init_DatabaseOld(Database* dba);
/*
 * Initializes the old ones!
 */

#endif
